@@ -6,7 +6,9 @@ import sys
 import tempfile
 import unittest
 
-from test.support import requires, import_module, verbose, SaveSignals
+from test.support import (requires, verbose, SaveSignals, cpython_only,
+                          check_disallow_instantiation)
+from test.support.import_helper import import_module
 
 # Optionally test curses module.  This currently requires that the
 # 'curses' resource be given on the regrtest command line using the -u
@@ -266,12 +268,7 @@ class TestCurses(unittest.TestCase):
         stdscr.echochar(b'A')
         stdscr.echochar(65)
         with self.assertRaises((UnicodeEncodeError, OverflowError)):
-            # Unicode is not fully supported yet, but at least it does
-            # not crash.
-            # It is supposed to fail because either the character is
-            # not encodable with the current encoding, or it is encoded to
-            # a multibyte sequence.
-            stdscr.echochar('\u0114')
+            stdscr.echochar('\u20ac')
         stdscr.echochar('A', curses.A_BOLD)
         self.assertIs(stdscr.is_wintouched(), False)
 
@@ -570,13 +567,12 @@ class TestCurses(unittest.TestCase):
     @requires_curses_window_meth('enclose')
     def test_enclose(self):
         win = curses.newwin(5, 15, 2, 5)
-        # TODO: Return bool instead of 1/0
-        self.assertTrue(win.enclose(2, 5))
-        self.assertFalse(win.enclose(1, 5))
-        self.assertFalse(win.enclose(2, 4))
-        self.assertTrue(win.enclose(6, 19))
-        self.assertFalse(win.enclose(7, 19))
-        self.assertFalse(win.enclose(6, 20))
+        self.assertIs(win.enclose(2, 5), True)
+        self.assertIs(win.enclose(1, 5), False)
+        self.assertIs(win.enclose(2, 4), False)
+        self.assertIs(win.enclose(6, 19), True)
+        self.assertIs(win.enclose(7, 19), False)
+        self.assertIs(win.enclose(6, 20), False)
 
     def test_putwin(self):
         win = curses.newwin(5, 12, 1, 2)
@@ -862,10 +858,13 @@ class TestCurses(unittest.TestCase):
         self.assertEqual(curses.getsyx(), (4, 5))
 
     def bad_colors(self):
-        return (-2**31 - 1, 2**31, -2**63 - 1, 2**63, 2**64)
+        return (-1, curses.COLORS, -2**31 - 1, 2**31, -2**63 - 1, 2**63, 2**64)
+
+    def bad_colors2(self):
+        return (curses.COLORS, 2**31, 2**63, 2**64)
 
     def bad_pairs(self):
-        return (-2**31 - 1, 2**31, -2**63 - 1, 2**63, 2**64)
+        return (-1, -2**31 - 1, 2**31, -2**63 - 1, 2**63, 2**64)
 
     def test_has_colors(self):
         self.assertIsInstance(curses.has_colors(), bool)
@@ -883,14 +882,11 @@ class TestCurses(unittest.TestCase):
     def test_color_content(self):
         self.assertEqual(curses.color_content(curses.COLOR_BLACK), (0, 0, 0))
         curses.color_content(0)
-        maxcolor = min(curses.COLORS - 1, SHORT_MAX)
+        maxcolor = curses.COLORS - 1
         curses.color_content(maxcolor)
 
         for color in self.bad_colors():
-            self.assertRaises(OverflowError, curses.color_content, color)
-        if curses.COLORS <= SHORT_MAX:
-            self.assertRaises(curses.error, curses.color_content, curses.COLORS)
-        self.assertRaises(curses.error, curses.color_content, -1)
+            self.assertRaises(ValueError, curses.color_content, color)
 
     @requires_colors
     def test_init_color(self):
@@ -908,7 +904,7 @@ class TestCurses(unittest.TestCase):
         curses.init_color(0, 1000, 1000, 1000)
         self.assertEqual(curses.color_content(0), (1000, 1000, 1000))
 
-        maxcolor = min(curses.COLORS - 1, SHORT_MAX)
+        maxcolor = curses.COLORS - 1
         old = curses.color_content(maxcolor)
         curses.init_color(maxcolor, *old)
         self.addCleanup(curses.init_color, maxcolor, *old)
@@ -916,17 +912,28 @@ class TestCurses(unittest.TestCase):
         self.assertEqual(curses.color_content(maxcolor), (0, 500, 1000))
 
         for color in self.bad_colors():
-            self.assertRaises(OverflowError, curses.init_color, color, 0, 0, 0)
-        if curses.COLORS <= SHORT_MAX:
-            self.assertRaises(curses.error, curses.init_color, curses.COLORS, 0, 0, 0)
-        self.assertRaises(curses.error, curses.init_color, -1, 0, 0, 0)
+            self.assertRaises(ValueError, curses.init_color, color, 0, 0, 0)
         for comp in (-1, 1001):
-            self.assertRaises(curses.error, curses.init_color, 0, comp, 0, 0)
-            self.assertRaises(curses.error, curses.init_color, 0, 0, comp, 0)
-            self.assertRaises(curses.error, curses.init_color, 0, 0, 0, comp)
+            self.assertRaises(ValueError, curses.init_color, 0, comp, 0, 0)
+            self.assertRaises(ValueError, curses.init_color, 0, 0, comp, 0)
+            self.assertRaises(ValueError, curses.init_color, 0, 0, 0, comp)
 
     def get_pair_limit(self):
-        return min(curses.COLOR_PAIRS, SHORT_MAX)
+        pair_limit = curses.COLOR_PAIRS
+        if hasattr(curses, 'ncurses_version'):
+            if curses.has_extended_color_support():
+                pair_limit += 2*curses.COLORS + 1
+            if (not curses.has_extended_color_support()
+                    or (6, 1) <= curses.ncurses_version < (6, 2)):
+                pair_limit = min(pair_limit, SHORT_MAX)
+            # If use_default_colors() is called, the upper limit of the extended
+            # range may be restricted, so we need to check if the limit is still
+            # correct
+            try:
+                curses.init_pair(pair_limit - 1, 0, 0)
+            except ValueError:
+                pair_limit = curses.COLOR_PAIRS
+        return pair_limit
 
     @requires_colors
     def test_pair_content(self):
@@ -939,8 +946,7 @@ class TestCurses(unittest.TestCase):
             curses.pair_content(maxpair)
 
         for pair in self.bad_pairs():
-            self.assertRaises(OverflowError, curses.pair_content, pair)
-        self.assertRaises(curses.error, curses.pair_content, -1)
+            self.assertRaises(ValueError, curses.pair_content, pair)
 
     @requires_colors
     def test_init_pair(self):
@@ -950,7 +956,7 @@ class TestCurses(unittest.TestCase):
 
         curses.init_pair(1, 0, 0)
         self.assertEqual(curses.pair_content(1), (0, 0))
-        maxcolor = min(curses.COLORS - 1, SHORT_MAX)
+        maxcolor = curses.COLORS - 1
         curses.init_pair(1, maxcolor, 0)
         self.assertEqual(curses.pair_content(1), (maxcolor, 0))
         curses.init_pair(1, 0, maxcolor)
@@ -961,14 +967,10 @@ class TestCurses(unittest.TestCase):
             self.assertEqual(curses.pair_content(maxpair), (0, 0))
 
         for pair in self.bad_pairs():
-            self.assertRaises(OverflowError, curses.init_pair, pair, 0, 0)
-        self.assertRaises(curses.error, curses.init_pair, -1, 0, 0)
-        for color in self.bad_colors():
-            self.assertRaises(OverflowError, curses.init_pair, 1, color, 0)
-            self.assertRaises(OverflowError, curses.init_pair, 1, 0, color)
-        if curses.COLORS <= SHORT_MAX:
-            self.assertRaises(curses.error, curses.init_pair, 1, curses.COLORS, 0)
-            self.assertRaises(curses.error, curses.init_pair, 1, 0, curses.COLORS)
+            self.assertRaises(ValueError, curses.init_pair, pair, 0, 0)
+        for color in self.bad_colors2():
+            self.assertRaises(ValueError, curses.init_pair, 1, color, 0)
+            self.assertRaises(ValueError, curses.init_pair, 1, 0, color)
 
     @requires_colors
     def test_color_attrs(self):
@@ -1045,11 +1047,13 @@ class TestCurses(unittest.TestCase):
         panel.set_userptr(A())
         panel.set_userptr(None)
 
+    @cpython_only
     @requires_curses_func('panel')
-    def test_new_curses_panel(self):
+    def test_disallow_instantiation(self):
+        # Ensure that the type disallows instantiation (bpo-43916)
         w = curses.newwin(10, 10)
         panel = curses.panel.new_panel(w)
-        self.assertRaises(TypeError, type(panel))
+        check_disallow_instantiation(self, type(panel))
 
     @requires_curses_func('is_term_resized')
     def test_is_term_resized(self):
@@ -1201,6 +1205,10 @@ class MiscTests(unittest.TestCase):
         self.assertGreaterEqual(v.minor, 0)
         self.assertGreaterEqual(v.patch, 0)
 
+    def test_has_extended_color_support(self):
+        r = curses.has_extended_color_support()
+        self.assertIsInstance(r, bool)
+
 
 class TestAscii(unittest.TestCase):
 
