@@ -2,7 +2,7 @@ import os
 import textwrap
 import unittest
 
-from test import support
+from test.support import os_helper
 from test.support.script_helper import assert_python_ok
 
 
@@ -13,8 +13,8 @@ class TestLLTrace(unittest.TestCase):
         # bpo-34113. The crash happened at the command line console of
         # debug Python builds with __ltrace__ enabled (only possible in console),
         # when the internal Python stack was negatively adjusted
-        with open(support.TESTFN, 'w') as fd:
-            self.addCleanup(os.unlink, support.TESTFN)
+        with open(os_helper.TESTFN, 'w', encoding='utf-8') as fd:
+            self.addCleanup(os_helper.unlink, os_helper.TESTFN)
             fd.write(textwrap.dedent("""\
             import code
 
@@ -25,7 +25,7 @@ class TestLLTrace(unittest.TestCase):
             print('unreachable if bug exists')
             """))
 
-            assert_python_ok(support.TESTFN)
+            assert_python_ok(os_helper.TESTFN)
 
 if __name__ == "__main__":
     unittest.main()
